@@ -16,7 +16,7 @@
 // under the License.
 
 //! An object store implementation for a local filesystem
-use std::fs::{metadata, symlink_metadata, File, Metadata, OpenOptions};
+use std::fs::{File, Metadata, OpenOptions, metadata, symlink_metadata};
 use std::io::{ErrorKind, Read, Seek, SeekFrom, Write};
 use std::ops::Range;
 use std::sync::Arc;
@@ -27,20 +27,20 @@ use std::{collections::VecDeque, path::PathBuf};
 use async_trait::async_trait;
 use bytes::Bytes;
 use chrono::{DateTime, Utc};
-use futures::{stream::BoxStream, StreamExt};
 use futures::{FutureExt, TryStreamExt};
+use futures::{StreamExt, stream::BoxStream};
 use parking_lot::Mutex;
 use url::Url;
 use walkdir::{DirEntry, WalkDir};
 
 use crate::{
-    maybe_spawn_blocking,
-    path::{absolute_path_to_url, Path},
-    util::InvalidGetRange,
     Attributes, GetOptions, GetResult, GetResultPayload, ListResult, MultipartUpload, ObjectMeta,
     ObjectStore, PutMode, PutMultipartOptions, PutOptions, PutPayload, PutResult, Result,
-    UploadPart,
+    UploadPart, maybe_spawn_blocking,
+    path::{Path, absolute_path_to_url},
+    util::InvalidGetRange,
 };
+use crate::{CopyMode, CopyOptions, ObjectStoreExt, RenameOptions, RenameTargetMode};
 
 /// A specialized `Error` for filesystem object store-related errors
 #[derive(Debug, thiserror::Error)]
@@ -190,7 +190,7 @@ impl From<Error> for super::Error {
 ///
 /// # Cross-Filesystem Copy
 ///
-/// [`LocalFileSystem::copy`] is implemented using [`std::fs::hard_link`], and therefore
+/// [`LocalFileSystem::copy_opts`] is implemented using [`std::fs::hard_link`], and therefore
 /// does not support copying across filesystem boundaries.
 ///
 #[derive(Debug)]
@@ -248,28 +248,7 @@ impl LocalFileSystem {
 
     /// Return an absolute filesystem path of the given file location
     pub fn path_to_filesystem(&self, location: &Path) -> Result<PathBuf> {
-        if !is_valid_file_path(location) {
-            let path = location.as_ref().into();
-            let error = Error::InvalidPath { path };
-            return Err(error.into());
-        }
-
-        let path = self.config.prefix_to_filesystem(location)?;
-
-        #[cfg(target_os = "windows")]
-        let path = {
-            let path = path.to_string_lossy();
-
-            // Assume the first char is the drive letter and the next is a colon.
-            let mut out = String::new();
-            let drive = &path[..2]; // The drive letter and colon (e.g., "C:")
-            let filepath = &path[2..].replace(':', "%3A"); // Replace subsequent colons
-            out.push_str(drive);
-            out.push_str(filepath);
-            PathBuf::from(out)
-        };
-
-        Ok(path)
+        self.config.path_to_filesystem(location)
     }
 
     /// Enable automatic cleanup of empty directories when deleting files
@@ -294,6 +273,32 @@ impl Config {
             .map_err(|_| Error::InvalidUrl { url }.into())
     }
 
+    /// Return an absolute filesystem path of the given file location
+    fn path_to_filesystem(&self, location: &Path) -> Result<PathBuf> {
+        if !is_valid_file_path(location) {
+            let path = location.as_ref().into();
+            let error = Error::InvalidPath { path };
+            return Err(error.into());
+        }
+
+        let path = self.prefix_to_filesystem(location)?;
+
+        #[cfg(target_os = "windows")]
+        let path = {
+            let path = path.to_string_lossy();
+
+            // Assume the first char is the drive letter and the next is a colon.
+            let mut out = String::new();
+            let drive = &path[..2]; // The drive letter and colon (e.g., "C:")
+            let filepath = &path[2..].replace(':', "%3A"); // Replace subsequent colons
+            out.push_str(drive);
+            out.push_str(filepath);
+            PathBuf::from(out)
+        };
+
+        Ok(path)
+    }
+
     /// Resolves the provided absolute filesystem path to a [`Path`] prefix
     fn filesystem_to_path(&self, location: &std::path::Path) -> Result<Path> {
         Ok(Path::from_absolute_path_with_base(
@@ -325,11 +330,17 @@ impl ObjectStore for LocalFileSystem {
         opts: PutOptions,
     ) -> Result<PutResult> {
         if matches!(opts.mode, PutMode::Update(_)) {
-            return Err(crate::Error::NotImplemented);
+            return Err(crate::Error::NotImplemented {
+                operation: "`put_opts` with mode `PutMode::Update`".into(),
+                implementer: self.to_string(),
+            });
         }
 
         if !opts.attributes.is_empty() {
-            return Err(crate::Error::NotImplemented);
+            return Err(crate::Error::NotImplemented {
+                operation: "`put_opts` with `opts.attributes` specified".into(),
+                implementer: self.to_string(),
+            });
         }
 
         let path = self.path_to_filesystem(location)?;
@@ -392,7 +403,10 @@ impl ObjectStore for LocalFileSystem {
         opts: PutMultipartOptions,
     ) -> Result<Box<dyn MultipartUpload>> {
         if !opts.attributes.is_empty() {
-            return Err(crate::Error::NotImplemented);
+            return Err(crate::Error::NotImplemented {
+                operation: "`put_multipart_opts` with `opts.attributes` specified".into(),
+                implementer: self.to_string(),
+            });
         }
 
         let dest = self.path_to_filesystem(location)?;
@@ -425,66 +439,41 @@ impl ObjectStore for LocalFileSystem {
         .await
     }
 
-    async fn get_range(&self, location: &Path, range: Range<u64>) -> Result<Bytes> {
-        let path = self.path_to_filesystem(location)?;
-        maybe_spawn_blocking(move || {
-            let (mut file, metadata) = open_file(&path)?;
-            read_range(&mut file, metadata.len(), &path, range)
-        })
-        .await
-    }
-
     async fn get_ranges(&self, location: &Path, ranges: &[Range<u64>]) -> Result<Vec<Bytes>> {
         let path = self.path_to_filesystem(location)?;
         let ranges = ranges.to_vec();
         maybe_spawn_blocking(move || {
             // Vectored IO might be faster
-            let (mut file, metadata) = open_file(&path)?;
+            let (mut file, _) = open_file(&path)?;
             ranges
                 .into_iter()
-                .map(|r| read_range(&mut file, metadata.len(), &path, r))
+                .map(|r| read_range(&mut file, &path, r))
                 .collect()
         })
         .await
     }
 
-    async fn delete(&self, location: &Path) -> Result<()> {
+    fn delete_stream(
+        &self,
+        locations: BoxStream<'static, Result<Path>>,
+    ) -> BoxStream<'static, Result<Path>> {
         let config = Arc::clone(&self.config);
-        let path = self.path_to_filesystem(location)?;
-        let automactic_cleanup = self.automatic_cleanup;
-        maybe_spawn_blocking(move || {
-            if let Err(e) = std::fs::remove_file(&path) {
-                Err(match e.kind() {
-                    ErrorKind::NotFound => Error::NotFound { path, source: e }.into(),
-                    _ => Error::UnableToDeleteFile { path, source: e }.into(),
+        let automatic_cleanup = self.automatic_cleanup;
+        locations
+            .map(move |location| {
+                let config = Arc::clone(&config);
+                maybe_spawn_blocking(move || {
+                    let location = location?;
+                    Self::delete_location(config, automatic_cleanup, &location)?;
+                    Ok(location)
                 })
-            } else if automactic_cleanup {
-                let root = &config.root;
-                let root = root
-                    .to_file_path()
-                    .map_err(|_| Error::InvalidUrl { url: root.clone() })?;
-
-                // here we will try to traverse up and delete an empty dir if possible until we reach the root or get an error
-                let mut parent = path.parent();
-
-                while let Some(loc) = parent {
-                    if loc != root && std::fs::remove_dir(loc).is_ok() {
-                        parent = loc.parent();
-                    } else {
-                        break;
-                    }
-                }
-
-                Ok(())
-            } else {
-                Ok(())
-            }
-        })
-        .await
+            })
+            .buffered(10)
+            .boxed()
     }
 
     fn list(&self, prefix: Option<&Path>) -> BoxStream<'static, Result<ObjectMeta>> {
-        self.list_with_maybe_offset(prefix, None)
+        Self::list_with_maybe_offset(Arc::clone(&self.config), prefix, None)
     }
 
     fn list_with_offset(
@@ -492,7 +481,7 @@ impl ObjectStore for LocalFileSystem {
         prefix: Option<&Path>,
         offset: &Path,
     ) -> BoxStream<'static, Result<ObjectMeta>> {
-        self.list_with_maybe_offset(prefix, Some(offset))
+        Self::list_with_maybe_offset(Arc::clone(&self.config), prefix, Some(offset))
     }
 
     async fn list_with_delimiter(&self, prefix: Option<&Path>) -> Result<ListResult> {
@@ -546,91 +535,171 @@ impl ObjectStore for LocalFileSystem {
         .await
     }
 
-    async fn copy(&self, from: &Path, to: &Path) -> Result<()> {
-        let from = self.path_to_filesystem(from)?;
-        let to = self.path_to_filesystem(to)?;
-        let mut id = 0;
-        // In order to make this atomic we:
-        //
-        // - hard link to a hidden temporary file
-        // - atomically rename this temporary file into place
-        //
-        // This is necessary because hard_link returns an error if the destination already exists
-        maybe_spawn_blocking(move || loop {
-            let staged = staged_upload_path(&to, &id.to_string());
-            match std::fs::hard_link(&from, &staged) {
-                Ok(_) => {
-                    return std::fs::rename(&staged, &to).map_err(|source| {
-                        let _ = std::fs::remove_file(&staged); // Attempt to clean up
-                        Error::UnableToCopyFile { from, to, source }.into()
-                    });
-                }
-                Err(source) => match source.kind() {
-                    ErrorKind::AlreadyExists => id += 1,
-                    ErrorKind::NotFound => match from.exists() {
-                        true => create_parent_dirs(&to, source)?,
-                        false => return Err(Error::NotFound { path: from, source }.into()),
-                    },
-                    _ => return Err(Error::UnableToCopyFile { from, to, source }.into()),
-                },
-            }
-        })
-        .await
-    }
+    async fn copy_opts(&self, from: &Path, to: &Path, options: CopyOptions) -> Result<()> {
+        let CopyOptions {
+            mode,
+            extensions: _,
+        } = options;
 
-    async fn rename(&self, from: &Path, to: &Path) -> Result<()> {
-        let from = self.path_to_filesystem(from)?;
-        let to = self.path_to_filesystem(to)?;
-        maybe_spawn_blocking(move || loop {
-            match std::fs::rename(&from, &to) {
-                Ok(_) => return Ok(()),
-                Err(source) => match source.kind() {
-                    ErrorKind::NotFound => match from.exists() {
-                        true => create_parent_dirs(&to, source)?,
-                        false => return Err(Error::NotFound { path: from, source }.into()),
-                    },
-                    _ => return Err(Error::UnableToCopyFile { from, to, source }.into()),
-                },
-            }
-        })
-        .await
-    }
-
-    async fn copy_if_not_exists(&self, from: &Path, to: &Path) -> Result<()> {
         let from = self.path_to_filesystem(from)?;
         let to = self.path_to_filesystem(to)?;
 
-        maybe_spawn_blocking(move || loop {
-            match std::fs::hard_link(&from, &to) {
-                Ok(_) => return Ok(()),
-                Err(source) => match source.kind() {
-                    ErrorKind::AlreadyExists => {
-                        return Err(Error::AlreadyExists {
-                            path: to.to_str().unwrap().to_string(),
-                            source,
+        match mode {
+            CopyMode::Overwrite => {
+                let mut id = 0;
+                // In order to make this atomic we:
+                //
+                // - hard link to a hidden temporary file
+                // - atomically rename this temporary file into place
+                //
+                // This is necessary because hard_link returns an error if the destination already exists
+                maybe_spawn_blocking(move || {
+                    loop {
+                        let staged = staged_upload_path(&to, &id.to_string());
+                        match std::fs::hard_link(&from, &staged) {
+                            Ok(_) => {
+                                return std::fs::rename(&staged, &to).map_err(|source| {
+                                    let _ = std::fs::remove_file(&staged); // Attempt to clean up
+                                    Error::UnableToCopyFile { from, to, source }.into()
+                                });
+                            }
+                            Err(source) => match source.kind() {
+                                ErrorKind::AlreadyExists => id += 1,
+                                ErrorKind::NotFound => match from.exists() {
+                                    true => create_parent_dirs(&to, source)?,
+                                    false => {
+                                        return Err(Error::NotFound { path: from, source }.into());
+                                    }
+                                },
+                                _ => {
+                                    return Err(Error::UnableToCopyFile { from, to, source }.into());
+                                }
+                            },
                         }
-                        .into())
                     }
-                    ErrorKind::NotFound => match from.exists() {
-                        true => create_parent_dirs(&to, source)?,
-                        false => return Err(Error::NotFound { path: from, source }.into()),
-                    },
-                    _ => return Err(Error::UnableToCopyFile { from, to, source }.into()),
-                },
+                })
+                .await
             }
-        })
-        .await
+            CopyMode::Create => {
+                maybe_spawn_blocking(move || {
+                    loop {
+                        match std::fs::hard_link(&from, &to) {
+                            Ok(_) => return Ok(()),
+                            Err(source) => match source.kind() {
+                                ErrorKind::AlreadyExists => {
+                                    return Err(Error::AlreadyExists {
+                                        path: to.to_str().unwrap().to_string(),
+                                        source,
+                                    }
+                                    .into());
+                                }
+                                ErrorKind::NotFound => match from.exists() {
+                                    true => create_parent_dirs(&to, source)?,
+                                    false => {
+                                        return Err(Error::NotFound { path: from, source }.into());
+                                    }
+                                },
+                                _ => {
+                                    return Err(Error::UnableToCopyFile { from, to, source }.into());
+                                }
+                            },
+                        }
+                    }
+                })
+                .await
+            }
+        }
+    }
+
+    async fn rename_opts(&self, from: &Path, to: &Path, options: RenameOptions) -> Result<()> {
+        let RenameOptions {
+            target_mode,
+            extensions,
+        } = options;
+
+        match target_mode {
+            // optimized implementation
+            RenameTargetMode::Overwrite => {
+                let from = self.path_to_filesystem(from)?;
+                let to = self.path_to_filesystem(to)?;
+                maybe_spawn_blocking(move || {
+                    loop {
+                        match std::fs::rename(&from, &to) {
+                            Ok(_) => return Ok(()),
+                            Err(source) => match source.kind() {
+                                ErrorKind::NotFound => match from.exists() {
+                                    true => create_parent_dirs(&to, source)?,
+                                    false => {
+                                        return Err(Error::NotFound { path: from, source }.into());
+                                    }
+                                },
+                                _ => {
+                                    return Err(Error::UnableToCopyFile { from, to, source }.into());
+                                }
+                            },
+                        }
+                    }
+                })
+                .await
+            }
+            // fall-back to copy & delete
+            RenameTargetMode::Create => {
+                self.copy_opts(
+                    from,
+                    to,
+                    CopyOptions {
+                        mode: CopyMode::Create,
+                        extensions,
+                    },
+                )
+                .await?;
+                self.delete(from).await?;
+                Ok(())
+            }
+        }
     }
 }
 
 impl LocalFileSystem {
+    fn delete_location(
+        config: Arc<Config>,
+        automatic_cleanup: bool,
+        location: &Path,
+    ) -> Result<()> {
+        let path = config.path_to_filesystem(location)?;
+        if let Err(e) = std::fs::remove_file(&path) {
+            Err(match e.kind() {
+                ErrorKind::NotFound => Error::NotFound { path, source: e }.into(),
+                _ => Error::UnableToDeleteFile { path, source: e }.into(),
+            })
+        } else if automatic_cleanup {
+            let root = &config.root;
+            let root = root
+                .to_file_path()
+                .map_err(|_| Error::InvalidUrl { url: root.clone() })?;
+
+            // here we will try to traverse up and delete an empty dir if possible until we reach the root or get an error
+            let mut parent = path.parent();
+
+            while let Some(loc) = parent {
+                if loc != root && std::fs::remove_dir(loc).is_ok() {
+                    parent = loc.parent();
+                } else {
+                    break;
+                }
+            }
+
+            Ok(())
+        } else {
+            Ok(())
+        }
+    }
+
     fn list_with_maybe_offset(
-        &self,
+        config: Arc<Config>,
         prefix: Option<&Path>,
         maybe_offset: Option<&Path>,
     ) -> BoxStream<'static, Result<ObjectMeta>> {
-        let config = Arc::clone(&self.config);
-
         let root_path = match prefix {
             Some(prefix) => match config.prefix_to_filesystem(prefix) {
                 Ok(path) => path,
@@ -899,14 +968,15 @@ pub(crate) fn chunked_stream(
     .boxed()
 }
 
-pub(crate) fn read_range(
-    file: &mut File,
-    file_len: u64,
-    path: &PathBuf,
-    range: Range<u64>,
-) -> Result<Bytes> {
+pub(crate) fn read_range(file: &mut File, path: &PathBuf, range: Range<u64>) -> Result<Bytes> {
+    let file_metadata = file.metadata().map_err(|e| Error::Metadata {
+        source: e.into(),
+        path: path.to_string_lossy().to_string(),
+    })?;
+
     // If none of the range is satisfiable we should error, e.g. if the start offset is beyond the
     // extents of the file
+    let file_len = file_metadata.len();
     if range.start >= file_len {
         return Err(Error::InvalidRange {
             source: InvalidGetRange::StartTooLarge {
@@ -1089,7 +1159,7 @@ mod tests {
     #[cfg(target_family = "unix")]
     use tempfile::NamedTempFile;
 
-    use crate::integration::*;
+    use crate::{ObjectStoreExt, integration::*};
 
     use super::*;
 
@@ -1613,7 +1683,10 @@ mod tests {
 
         let b = Path::parse("bar#123").unwrap();
         let err = integration.get(&b).await.unwrap_err().to_string();
-        assert_eq!(err, "Generic LocalFileSystem error: Filenames containing trailing '/#\\d+/' are not supported: bar#123");
+        assert_eq!(
+            err,
+            "Generic LocalFileSystem error: Filenames containing trailing '/#\\d+/' are not supported: bar#123"
+        );
 
         let c = Path::parse("foo#123.txt").unwrap();
         integration.put(&c, "test".into()).await.unwrap();
@@ -1681,7 +1754,7 @@ mod not_wasm_tests {
     use tempfile::TempDir;
 
     use crate::local::LocalFileSystem;
-    use crate::{ObjectStore, Path, PutPayload};
+    use crate::{ObjectStoreExt, Path, PutPayload};
 
     #[tokio::test]
     async fn test_cleanup_intermediate_files() {
@@ -1718,7 +1791,7 @@ mod unix_test {
     use tempfile::TempDir;
 
     use crate::local::LocalFileSystem;
-    use crate::{ObjectStore, Path};
+    use crate::{ObjectStoreExt, Path};
 
     #[tokio::test]
     async fn test_fifo() {
