@@ -23,17 +23,17 @@ use std::sync::Arc;
 use async_trait::async_trait;
 use bytes::Bytes;
 use chrono::{DateTime, Utc};
-use futures::{stream::BoxStream, StreamExt};
+use futures::{StreamExt, stream::BoxStream};
 use parking_lot::RwLock;
 
 use crate::multipart::{MultipartStore, PartId};
 use crate::util::InvalidGetRange;
 use crate::{
-    path::Path, Attributes, GetRange, GetResult, GetResultPayload, ListResult, MultipartId,
-    MultipartUpload, ObjectMeta, ObjectStore, PutMode, PutMultipartOptions, PutOptions, PutResult,
-    Result, UpdateVersion, UploadPart,
+    Attributes, GetRange, GetResult, GetResultPayload, ListResult, MultipartId, MultipartUpload,
+    ObjectMeta, ObjectStore, PutMode, PutMultipartOptions, PutOptions, PutResult, Result,
+    UpdateVersion, UploadPart, path::Path,
 };
-use crate::{GetOptions, PutPayload};
+use crate::{CopyMode, CopyOptions, GetOptions, PutPayload};
 
 /// A specialized `Error` for in-memory object store-related errors
 #[derive(Debug, thiserror::Error)]
@@ -294,21 +294,18 @@ impl ObjectStore for InMemory {
             .collect()
     }
 
-    async fn head(&self, location: &Path) -> Result<ObjectMeta> {
-        let entry = self.entry(location)?;
-
-        Ok(ObjectMeta {
-            location: location.clone(),
-            last_modified: entry.last_modified,
-            size: entry.data.len() as u64,
-            e_tag: Some(entry.e_tag.to_string()),
-            version: None,
-        })
-    }
-
-    async fn delete(&self, location: &Path) -> Result<()> {
-        self.storage.write().map.remove(location);
-        Ok(())
+    fn delete_stream(
+        &self,
+        locations: BoxStream<'static, Result<Path>>,
+    ) -> BoxStream<'static, Result<Path>> {
+        let storage = Arc::clone(&self.storage);
+        locations
+            .map(move |location| {
+                let location = location?;
+                storage.write().map.remove(&location);
+                Ok(location)
+            })
+            .boxed()
     }
 
     fn list(&self, prefix: Option<&Path>) -> BoxStream<'static, Result<ObjectMeta>> {
@@ -389,24 +386,30 @@ impl ObjectStore for InMemory {
         })
     }
 
-    async fn copy(&self, from: &Path, to: &Path) -> Result<()> {
-        let entry = self.entry(from)?;
-        self.storage
-            .write()
-            .insert(to, entry.data, entry.attributes);
-        Ok(())
-    }
+    async fn copy_opts(&self, from: &Path, to: &Path, options: CopyOptions) -> Result<()> {
+        let CopyOptions {
+            mode,
+            extensions: _,
+        } = options;
 
-    async fn copy_if_not_exists(&self, from: &Path, to: &Path) -> Result<()> {
         let entry = self.entry(from)?;
         let mut storage = self.storage.write();
-        if storage.map.contains_key(to) {
-            return Err(Error::AlreadyExists {
-                path: to.to_string(),
+
+        match mode {
+            CopyMode::Overwrite => {
+                storage.insert(to, entry.data, entry.attributes);
+            }
+            CopyMode::Create => {
+                if storage.map.contains_key(to) {
+                    return Err(Error::AlreadyExists {
+                        path: to.to_string(),
+                    }
+                    .into());
+                }
+                storage.insert(to, entry.data, entry.attributes);
             }
-            .into());
         }
-        storage.insert(to, entry.data, entry.attributes);
+
         Ok(())
     }
 }
@@ -536,7 +539,7 @@ impl MultipartUpload for InMemoryUpload {
 
 #[cfg(test)]
 mod tests {
-    use crate::integration::*;
+    use crate::{ObjectStoreExt, integration::*};
 
     use super::*;
 
